#! /usr/bin/env python
# coding: utf-8

from flask_helper import Flask2 as MFlask
from flask_helper import FlaskCrossDomain as CrossDomain


__author__ = 'meisanggou'


JYFlaskCrossDomain = CrossDomain
